# Copyright 2009 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A fake shutil module implementation that uses fake_filesystem for
unit tests.
Note that only `shutildisk_usage()` is faked, the rest of the functions shall
work fine with the fake file system if `os`/`os.path` are patched.

:Includes:
  FakeShutil: Uses a FakeFilesystem to provide a fake replacement for the
    shutil module.

:Usage:
  The fake implementation is automatically involved if using
  `fake_filesystem_unittest.TestCase`, pytest fs fixture,
  or directly `Patcher`.
"""

import shutil


class FakeShutilModule:
    """Uses a FakeFilesystem to provide a fake replacement for shutil module.
    """

    @staticmethod
    def dir():
        """Return the list of patched function names. Used for patching
        functions imported from the module.
        """
        return 'disk_usage',

    def __init__(self, filesystem):
        """Construct fake shutil module using the fake filesystem.

        Args:
          filesystem:  FakeFilesystem used to provide file system information
        """
        self.filesystem = filesystem
        self._shutil_module = shutil

    def disk_usage(self, path):
        """Return the total, used and free disk space in bytes as named tuple
        or placeholder holder values simulating unlimited space if not set.

        Args:
          path: defines the filesystem device which is queried
        """
        return self.filesystem.get_disk_usage(path)

    def __getattr__(self, name):
        """Forwards any non-faked calls to the standard shutil module."""
        return getattr(self._shutil_module, name)



# Copyright 2014 Altera Corporation. All Rights Reserved.
# Copyright 2015-2017 John McGehee
# Author: John McGehee
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Test the :py:class`pyfakefs.fake_filesystem_unittest.TestCase` base class.
"""
import glob
import io
import multiprocessing
import os
import shutil
import sys
import tempfile
import unittest
from distutils.dir_util import copy_tree, remove_tree
from unittest import TestCase

import pyfakefs.tests.import_as_example
from pyfakefs import fake_filesystem_unittest, fake_filesystem
from pyfakefs.extra_packages import pathlib
from pyfakefs.fake_filesystem_unittest import Patcher, Pause, patchfs
from pyfakefs.tests.fixtures import module_with_attributes


class TestPatcher(TestCase):
    def test_context_manager(self):
        with Patcher() as patcher:
            patcher.fs.create_file('/foo/bar', contents='test')
            with open('/foo/bar') as f:
                contents = f.read()
            self.assertEqual('test', contents)

    @patchfs
    def test_context_decorator(self, fs):
        fs.create_file('/foo/bar', contents='test')
        with open('/foo/bar') as f:
            contents = f.read()
        self.assertEqual('test', contents)


class TestPyfakefsUnittestBase(fake_filesystem_unittest.TestCase):
    def setUp(self):
        """Set up the fake file system"""
        self.setUpPyfakefs()


class TestPyfakefsUnittest(TestPyfakefsUnittestBase):  # pylint: disable=R0904
    """Test the `pyfakefs.fake_filesystem_unittest.TestCase` base class."""

    def test_open(self):
        """Fake `open()` function is bound"""
        self.assertFalse(os.path.exists('/fake_file.txt'))
        with open('/fake_file.txt', 'w') as f:
            f.write("This test file was created using the open() function.\n")
        self.assertTrue(self.fs.exists('/fake_file.txt'))
        with open('/fake_file.txt') as f:
            content = f.read()
        self.assertEqual(content, 'This test file was created using the '
                                  'open() function.\n')

    def test_io_open(self):
        """Fake io module is bound"""
        self.assertFalse(os.path.exists('/fake_file.txt'))
        with io.open('/fake_file.txt', 'w') as f:
            f.write("This test file was created using the"
                    " io.open() function.\n")
        self.assertTrue(self.fs.exists('/fake_file.txt'))
        with open('/fake_file.txt') as f:
            content = f.read()
        self.assertEqual(content, 'This test file was created using the '
                                  'io.open() function.\n')

    def test_os(self):
        """Fake os module is bound"""
        self.assertFalse(self.fs.exists('/test/dir1/dir2'))
        os.makedirs('/test/dir1/dir2')
        self.assertTrue(self.fs.exists('/test/dir1/dir2'))

    def test_glob(self):
        """Fake glob module is bound"""
        is_windows = sys.platform.startswith('win')
        self.assertEqual(glob.glob('/test/dir1/dir*'),
                         [])
        self.fs.create_dir('/test/dir1/dir2a')
        matching_paths = glob.glob('/test/dir1/dir*')
        if is_windows:
            self.assertEqual(matching_paths, [r'\test\dir1\dir2a'])
        else:
            self.assertEqual(matching_paths, ['/test/dir1/dir2a'])
        self.fs.create_dir('/test/dir1/dir2b')
        matching_paths = sorted(glob.glob('/test/dir1/dir*'))
        if is_windows:
            self.assertEqual(matching_paths,
                             [r'\test\dir1\dir2a', r'\test\dir1\dir2b'])
        else:
            self.assertEqual(matching_paths,
                             ['/test/dir1/dir2a', '/test/dir1/dir2b'])

    def test_shutil(self):
        """Fake shutil module is bound"""
        self.fs.create_dir('/test/dir1/dir2a')
        self.fs.create_dir('/test/dir1/dir2b')
        self.assertTrue(self.fs.exists('/test/dir1/dir2b'))
        self.assertTrue(self.fs.exists('/test/dir1/dir2a'))

        shutil.rmtree('/test/dir1')
        self.assertFalse(self.fs.exists('/test/dir1'))

    def test_fakepathlib(self):
        with pathlib.Path('/fake_file.txt') as p:
            with p.open('w') as f:
                f.write('text')
        is_windows = sys.platform.startswith('win')
        if is_windows:
            self.assertTrue(self.fs.exists(r'\fake_file.txt'))
        else:
            self.assertTrue(self.fs.exists('/fake_file.txt'))


class TestPatchingImports(TestPyfakefsUnittestBase):
    def test_import_as_other_name(self):
        file_path = '/foo/bar/baz'
        self.fs.create_file(file_path)
        self.assertTrue(self.fs.exists(file_path))
        self.assertTrue(
            pyfakefs.tests.import_as_example.check_if_exists1(file_path))

    def test_import_path_from_os(self):
        """Make sure `from os import path` patches `path`."""
        file_path = '/foo/bar/baz'
        self.fs.create_file(file_path)
        self.assertTrue(self.fs.exists(file_path))
        self.assertTrue(
            pyfakefs.tests.import_as_example.check_if_exists2(file_path))

    def test_import_path_from_pathlib(self):
        file_path = '/foo/bar'
        self.fs.create_dir(file_path)
        self.assertTrue(
            pyfakefs.tests.import_as_example.check_if_exists3(file_path))

    def test_import_function_from_os_path(self):
        file_path = '/foo/bar'
        self.fs.create_dir(file_path)
        self.assertTrue(
            pyfakefs.tests.import_as_example.check_if_exists5(file_path))

    def test_import_function_from_os_path_as_other_name(self):
        file_path = '/foo/bar'
        self.fs.create_dir(file_path)
        self.assertTrue(
            pyfakefs.tests.import_as_example.check_if_exists6(file_path))

    def test_import_function_from_os(self):
        file_path = '/foo/bar'
        self.fs.create_file(file_path, contents=b'abc')
        stat_result = pyfakefs.tests.import_as_example.file_stat1(file_path)
        self.assertEqual(3, stat_result.st_size)

    def test_import_function_from_os_as_other_name(self):
        file_path = '/foo/bar'
        self.fs.create_file(file_path, contents=b'abc')
        stat_result = pyfakefs.tests.import_as_example.file_stat2(file_path)
        self.assertEqual(3, stat_result.st_size)

    def test_import_open_as_other_name(self):
        file_path = '/foo/bar'
        self.fs.create_file(file_path, contents=b'abc')
        contents = pyfakefs.tests.import_as_example.file_contents1(file_path)
        self.assertEqual('abc', contents)

    def test_import_io_open_as_other_name(self):
        file_path = '/foo/bar'
        self.fs.create_file(file_path, contents=b'abc')
        contents = pyfakefs.tests.import_as_example.file_contents2(file_path)
        self.assertEqual('abc', contents)


class TestPatchingDefaultArgs(TestPyfakefsUnittestBase):
    def test_path_exists_as_default_arg_in_function(self):
        file_path = '/foo/bar'
        self.fs.create_dir(file_path)
        self.assertTrue(
            pyfakefs.tests.import_as_example.check_if_exists4(file_path))

    def test_path_exists_as_default_arg_in_method(self):
        file_path = '/foo/bar'
        self.fs.create_dir(file_path)
        sut = pyfakefs.tests.import_as_example.TestDefaultArg()
        self.assertTrue(sut.check_if_exists(file_path))


class TestAttributesWithFakeModuleNames(TestPyfakefsUnittestBase):
    """Test that module attributes with names like `path` or `io` are not
    stubbed out.
    """

    def test_attributes(self):
        """Attributes of module under test are not patched"""
        self.assertEqual(module_with_attributes.os, 'os attribute value')
        self.assertEqual(module_with_attributes.path, 'path attribute value')
        self.assertEqual(module_with_attributes.pathlib,
                         'pathlib attribute value')
        self.assertEqual(module_with_attributes.shutil,
                         'shutil attribute value')
        self.assertEqual(module_with_attributes.io, 'io attribute value')


import math as path  # noqa: E402 wanted import not at top


class TestPathNotPatchedIfNotOsPath(TestPyfakefsUnittestBase):
    """Tests that `path` is not patched if it is not `os.path`.
       An own path module (in this case an alias to math) can be imported
       and used.
    """

    def test_own_path_module(self):
        self.assertEqual(2, path.floor(2.5))


class FailedPatchingTest(TestPyfakefsUnittestBase):
    """Negative tests: make sure the tests for `modules_to_reload` and
    `modules_to_patch` fail if not providing the arguments.
    """

    @unittest.expectedFailure
    def test_system_stat(self):
        file_path = '/foo/bar'
        self.fs.create_file(file_path, contents=b'test')
        self.assertEqual(
            4, pyfakefs.tests.import_as_example.system_stat(file_path).st_size)


class ReloadModuleTest(fake_filesystem_unittest.TestCase):
    """Make sure that reloading a module allows patching of classes not
    patched automatically.
    """

    def setUp(self):
        """Set up the fake file system"""
        self.setUpPyfakefs(
            modules_to_reload=[pyfakefs.tests.import_as_example])


class NoSkipNamesTest(fake_filesystem_unittest.TestCase):
    """Reference test for additional_skip_names tests:
     make sure that the module is patched by default."""

    def test_path_exists(self):
        self.assertTrue(
            pyfakefs.tests.import_as_example.exists_this_file())


class AdditionalSkipNamesTest(fake_filesystem_unittest.TestCase):
    """Make sure that modules in additional_skip_names are not patched.
    Passes module name to `additional_skip_names`."""

    def setUp(self):
        self.setUpPyfakefs(
            additional_skip_names=['pyfakefs.tests.import_as_example'])

    def test_path_exists(self):
        self.assertFalse(
            pyfakefs.tests.import_as_example.exists_this_file())


class AdditionalSkipNamesModuleTest(fake_filesystem_unittest.TestCase):
    """Make sure that modules in additional_skip_names are not patched.
    Passes module to `additional_skip_names`."""

    def setUp(self):
        self.setUpPyfakefs(
            additional_skip_names=[pyfakefs.tests.import_as_example])

    def test_path_exists(self):
        self.assertFalse(
            pyfakefs.tests.import_as_example.exists_this_file())


class FakeExampleModule:
    """Used to patch a function that uses system-specific functions that
    cannot be patched automatically."""
    _orig_module = pyfakefs.tests.import_as_example

    def __init__(self, fs):
        pass

    def system_stat(self, filepath):
        return os.stat(filepath)

    def __getattr__(self, name):
        """Forwards any non-faked calls to the standard module."""
        return getattr(self._orig_module, name)


class PatchModuleTest(fake_filesystem_unittest.TestCase):
    """Make sure that reloading a module allows patching of classes not
    patched automatically.
    """

    def setUp(self):
        """Set up the fake file system"""
        self.setUpPyfakefs(
            modules_to_patch={
                'pyfakefs.tests.import_as_example': FakeExampleModule})

    def test_system_stat(self):
        file_path = '/foo/bar'
        self.fs.create_file(file_path, contents=b'test')
        self.assertEqual(
            4, pyfakefs.tests.import_as_example.system_stat(file_path).st_size)


class PatchModuleTestUsingDecorator(unittest.TestCase):
    """Make sure that reloading a module allows patching of classes not
    patched automatically - use patchfs decorator with parameter.
    """

    @patchfs
    @unittest.expectedFailure
    def test_system_stat_failing(self, fs):
        file_path = '/foo/bar'
        fs.create_file(file_path, contents=b'test')
        self.assertEqual(
            4, pyfakefs.tests.import_as_example.system_stat(file_path).st_size)

    @patchfs(modules_to_patch={
        'pyfakefs.tests.import_as_example': FakeExampleModule})
    def test_system_stat(self, fs):
        file_path = '/foo/bar'
        fs.create_file(file_path, contents=b'test')
        self.assertEqual(
            4, pyfakefs.tests.import_as_example.system_stat(file_path).st_size)


class NoRootUserTest(fake_filesystem_unittest.TestCase):
    """Test allow_root_user argument to setUpPyfakefs."""

    def setUp(self):
        self.setUpPyfakefs(allow_root_user=False)

    def test_non_root_behavior(self):
        """Check that fs behaves as non-root user regardless of actual
        user rights.
        """
        self.fs.is_windows_fs = False
        dir_path = '/foo/bar'
        self.fs.create_dir(dir_path, perm_bits=0o555)
        file_path = dir_path + 'baz'
        with self.assertRaises(OSError):
            self.fs.create_file(file_path)

        file_path = '/baz'
        self.fs.create_file(file_path)
        os.chmod(file_path, 0o400)
        with self.assertRaises(OSError):
            open(file_path, 'w')


class PauseResumeTest(TestPyfakefsUnittestBase):
    def test_pause_resume(self):
        fake_temp_file = tempfile.NamedTemporaryFile()
        self.assertTrue(self.fs.exists(fake_temp_file.name))
        self.assertTrue(os.path.exists(fake_temp_file.name))
        self.pause()
        self.assertTrue(self.fs.exists(fake_temp_file.name))
        self.assertFalse(os.path.exists(fake_temp_file.name))
        real_temp_file = tempfile.NamedTemporaryFile()
        self.assertFalse(self.fs.exists(real_temp_file.name))
        self.assertTrue(os.path.exists(real_temp_file.name))
        self.resume()
        self.assertFalse(os.path.exists(real_temp_file.name))
        self.assertTrue(os.path.exists(fake_temp_file.name))

    def test_pause_resume_fs(self):
        fake_temp_file = tempfile.NamedTemporaryFile()
        self.assertTrue(self.fs.exists(fake_temp_file.name))
        self.assertTrue(os.path.exists(fake_temp_file.name))
        # resume does nothing if not paused
        self.fs.resume()
        self.assertTrue(os.path.exists(fake_temp_file.name))
        self.fs.pause()
        self.assertTrue(self.fs.exists(fake_temp_file.name))
        self.assertFalse(os.path.exists(fake_temp_file.name))
        real_temp_file = tempfile.NamedTemporaryFile()
        self.assertFalse(self.fs.exists(real_temp_file.name))
        self.assertTrue(os.path.exists(real_temp_file.name))
        # pause does nothing if already paused
        self.fs.pause()
        self.assertFalse(self.fs.exists(real_temp_file.name))
        self.assertTrue(os.path.exists(real_temp_file.name))
        self.fs.resume()
        self.assertFalse(os.path.exists(real_temp_file.name))
        self.assertTrue(os.path.exists(fake_temp_file.name))

    def test_pause_resume_contextmanager(self):
        fake_temp_file = tempfile.NamedTemporaryFile()
        self.assertTrue(self.fs.exists(fake_temp_file.name))
        self.assertTrue(os.path.exists(fake_temp_file.name))
        with Pause(self):
            self.assertTrue(self.fs.exists(fake_temp_file.name))
            self.assertFalse(os.path.exists(fake_temp_file.name))
            real_temp_file = tempfile.NamedTemporaryFile()
            self.assertFalse(self.fs.exists(real_temp_file.name))
            self.assertTrue(os.path.exists(real_temp_file.name))
        self.assertFalse(os.path.exists(real_temp_file.name))
        self.assertTrue(os.path.exists(fake_temp_file.name))

    def test_pause_resume_fs_contextmanager(self):
        fake_temp_file = tempfile.NamedTemporaryFile()
        self.assertTrue(self.fs.exists(fake_temp_file.name))
        self.assertTrue(os.path.exists(fake_temp_file.name))
        with Pause(self.fs):
            self.assertTrue(self.fs.exists(fake_temp_file.name))
            self.assertFalse(os.path.exists(fake_temp_file.name))
            real_temp_file = tempfile.NamedTemporaryFile()
            self.assertFalse(self.fs.exists(real_temp_file.name))
            self.assertTrue(os.path.exists(real_temp_file.name))
        self.assertFalse(os.path.exists(real_temp_file.name))
        self.assertTrue(os.path.exists(fake_temp_file.name))

    def test_pause_resume_without_patcher(self):
        fs = fake_filesystem.FakeFilesystem()
        with self.assertRaises(RuntimeError):
            fs.resume()


class PauseResumePatcherTest(fake_filesystem_unittest.TestCase):
    def test_pause_resume(self):
        with Patcher() as p:
            fake_temp_file = tempfile.NamedTemporaryFile()
            self.assertTrue(p.fs.exists(fake_temp_file.name))
            self.assertTrue(os.path.exists(fake_temp_file.name))
            p.pause()
            self.assertTrue(p.fs.exists(fake_temp_file.name))
            self.assertFalse(os.path.exists(fake_temp_file.name))
            real_temp_file = tempfile.NamedTemporaryFile()
            self.assertFalse(p.fs.exists(real_temp_file.name))
            self.assertTrue(os.path.exists(real_temp_file.name))
            p.resume()
            self.assertFalse(os.path.exists(real_temp_file.name))
            self.assertTrue(os.path.exists(fake_temp_file.name))

    def test_pause_resume_contextmanager(self):
        with Patcher() as p:
            fake_temp_file = tempfile.NamedTemporaryFile()
            self.assertTrue(p.fs.exists(fake_temp_file.name))
            self.assertTrue(os.path.exists(fake_temp_file.name))
            with Pause(p):
                self.assertTrue(p.fs.exists(fake_temp_file.name))
                self.assertFalse(os.path.exists(fake_temp_file.name))
                real_temp_file = tempfile.NamedTemporaryFile()
                self.assertFalse(p.fs.exists(real_temp_file.name))
                self.assertTrue(os.path.exists(real_temp_file.name))
            self.assertFalse(os.path.exists(real_temp_file.name))
            self.assertTrue(os.path.exists(fake_temp_file.name))


class TestCopyOrAddRealFile(TestPyfakefsUnittestBase):
    """Tests the `fake_filesystem_unittest.TestCase.copyRealFile()` method.
    Note that `copyRealFile()` is deprecated in favor of
    `FakeFilesystem.add_real_file()`.
    """
    filepath = None

    @classmethod
    def setUpClass(cls):
        filename = __file__
        if filename.endswith('.pyc'):  # happens on windows / py27
            filename = filename[:-1]
        cls.filepath = os.path.abspath(filename)
        with open(cls.filepath) as f:
            cls.real_string_contents = f.read()
        with open(cls.filepath, 'rb') as f:
            cls.real_byte_contents = f.read()
        cls.real_stat = os.stat(cls.filepath)

    @unittest.skipIf(sys.platform == 'darwin', 'Different copy behavior')
    def test_copy_real_file(self):
        """Typical usage of deprecated copyRealFile()"""
        # Use this file as the file to be copied to the fake file system
        fake_file = self.copyRealFile(self.filepath)

        self.assertTrue(
            'class TestCopyOrAddRealFile(TestPyfakefsUnittestBase)'
            in self.real_string_contents,
            'Verify real file string contents')
        self.assertTrue(
            b'class TestCopyOrAddRealFile(TestPyfakefsUnittestBase)'
            in self.real_byte_contents,
            'Verify real file byte contents')

        # note that real_string_contents may differ to fake_file.contents
        # due to newline conversions in open()
        self.assertEqual(fake_file.byte_contents, self.real_byte_contents)

        self.assertEqual(oct(fake_file.st_mode), oct(self.real_stat.st_mode))
        self.assertEqual(fake_file.st_size, self.real_stat.st_size)
        self.assertAlmostEqual(fake_file.st_ctime,
                               self.real_stat.st_ctime, places=5)
        self.assertAlmostEqual(fake_file.st_atime,
                               self.real_stat.st_atime, places=5)
        self.assertLess(fake_file.st_atime, self.real_stat.st_atime + 10)
        self.assertAlmostEqual(fake_file.st_mtime,
                               self.real_stat.st_mtime, places=5)
        self.assertEqual(fake_file.st_uid, self.real_stat.st_uid)
        self.assertEqual(fake_file.st_gid, self.real_stat.st_gid)

    def test_copy_real_file_deprecated_arguments(self):
        """Deprecated copyRealFile() arguments"""
        self.assertFalse(self.fs.exists(self.filepath))
        # Specify redundant fake file path
        self.copyRealFile(self.filepath, self.filepath)
        self.assertTrue(self.fs.exists(self.filepath))

        # Test deprecated argument values
        with self.assertRaises(ValueError):
            self.copyRealFile(self.filepath, '/different/filename')
        with self.assertRaises(ValueError):
            self.copyRealFile(self.filepath, create_missing_dirs=False)

    def test_add_real_file(self):
        """Add a real file to the fake file system to be read on demand"""

        # this tests only the basic functionality inside a unit test, more
        # thorough tests are done in
        # fake_filesystem_test.RealFileSystemAccessTest
        fake_file = self.fs.add_real_file(self.filepath)
        self.assertTrue(self.fs.exists(self.filepath))
        self.assertIsNone(fake_file._byte_contents)
        self.assertEqual(self.real_byte_contents, fake_file.byte_contents)

    def test_add_real_directory(self):
        """Add a real directory and the contained files to the fake file system
        to be read on demand"""

        # This tests only the basic functionality inside a unit test,
        # more thorough tests are done in
        # fake_filesystem_test.RealFileSystemAccessTest.
        # Note: this test fails (add_real_directory raises) if 'genericpath'
        # is not added to SKIPNAMES
        real_dir_path = os.path.split(os.path.dirname(self.filepath))[0]
        self.fs.add_real_directory(real_dir_path)
        self.assertTrue(self.fs.exists(real_dir_path))
        self.assertTrue(self.fs.exists(
            os.path.join(real_dir_path, 'fake_filesystem.py')))

    def test_add_real_directory_with_backslash(self):
        """Add a real directory ending with a path separator."""
        real_dir_path = os.path.split(os.path.dirname(self.filepath))[0]
        self.fs.add_real_directory(real_dir_path + os.sep)
        self.assertTrue(self.fs.exists(real_dir_path))
        self.assertTrue(self.fs.exists(
            os.path.join(real_dir_path, 'fake_filesystem.py')))


class TestPyfakefsTestCase(unittest.TestCase):
    def setUp(self):
        class TestTestCase(fake_filesystem_unittest.TestCase):
            def runTest(self):
                pass

        self.test_case = TestTestCase('runTest')

    def test_test_case_type(self):
        self.assertIsInstance(self.test_case, unittest.TestCase)

        self.assertIsInstance(self.test_case,
                              fake_filesystem_unittest.TestCaseMixin)


class TestTempFileReload(unittest.TestCase):
    """Regression test for #356 to make sure that reloading the tempfile
    does not affect other tests."""

    def test_fakefs(self):
        with Patcher() as patcher:
            patcher.fs.create_file('/mytempfile', contents='abcd')

    def test_value(self):
        v = multiprocessing.Value('I', 0)
        self.assertEqual(v.value, 0)


class TestPyfakefsTestCaseMixin(unittest.TestCase,
                                fake_filesystem_unittest.TestCaseMixin):
    def test_set_up_pyfakefs(self):
        self.setUpPyfakefs()

        self.assertTrue(hasattr(self, 'fs'))
        self.assertIsInstance(self.fs, fake_filesystem.FakeFilesystem)


class TestShutilWithZipfile(fake_filesystem_unittest.TestCase):
    """Regression test for #427."""

    def setUp(self):
        self.setUpPyfakefs()
        self.fs.create_file('foo/bar')

    def test_a(self):
        shutil.make_archive('archive', 'zip', root_dir='foo')

    def test_b(self):
        # used to fail because 'bar' could not be found
        shutil.make_archive('archive', 'zip', root_dir='foo')


class TestDistutilsCopyTree(fake_filesystem_unittest.TestCase):
    """Regression test for #501."""

    def setUp(self):
        self.setUpPyfakefs()
        self.fs.create_dir("./test/subdir/")
        self.fs.create_dir("./test/subdir2/")
        self.fs.create_file("./test2/subdir/1.txt")

    def test_file_copied(self):
        copy_tree("./test2/", "./test/")
        remove_tree("./test2/")

        self.assertTrue(os.path.isfile('./test/subdir/1.txt'))
        self.assertFalse(os.path.isdir('./test2/'))

    def test_file_copied_again(self):
        # used to fail because 'test2' could not be found
        self.assertTrue(os.path.isfile('./test2/subdir/1.txt'))

        copy_tree("./test2/", "./test/")
        remove_tree("./test2/")

        self.assertTrue(os.path.isfile('./test/subdir/1.txt'))
        self.assertFalse(os.path.isdir('./test2/'))


class PathlibTest(TestCase):
    """Regression test for #527"""

    @patchfs
    def test_cwd(self, fs):
        """Make sure fake file system is used for os in pathlib"""
        self.assertEqual(os.path.sep, str(pathlib.Path.cwd()))
        dot_abs = pathlib.Path(".").absolute()
        self.assertEqual(os.path.sep, str(dot_abs))
        self.assertTrue(dot_abs.exists())


if __name__ == "__main__":
    unittest.main()



# Copyright 2009 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Test that FakeFilesystem calls work identically to a real filesystem."""
# pylint: disable-all

import os
import shutil
import sys
import tempfile
import time
import unittest

from pyfakefs import fake_filesystem


def sep(path):
    """Converts slashes in the path to the architecture's path seperator."""
    if isinstance(path, str):
        return path.replace('/', os.sep)
    return path


def _get_errno(raised_error):
    if raised_error is not None:
        try:
            return raised_error.errno
        except AttributeError:
            pass


class TestCase(unittest.TestCase):
    is_windows = sys.platform.startswith('win')
    _FAKE_FS_BASE = sep('/fakefs')


class FakeFilesystemVsRealTest(TestCase):
    def _paths(self, path):
        """For a given path, return paths in the real and fake filesystems."""
        if not path:
            return None, None
        return (os.path.join(self.real_base, path),
                os.path.join(self.fake_base, path))

    def _create_test_file(self, file_type, path, contents=None):
        """Create a dir, file, or link in both the real fs and the fake."""
        path = sep(path)
        self._created_files.append([file_type, path, contents])
        real_path, fake_path = self._paths(path)
        if file_type == 'd':
            os.mkdir(real_path)
            self.fake_os.mkdir(fake_path)
        if file_type == 'f':
            fh = open(real_path, 'w')
            fh.write(contents or '')
            fh.close()
            fh = self.fake_open(fake_path, 'w')
            fh.write(contents or '')
            fh.close()
        # b for binary file
        if file_type == 'b':
            fh = open(real_path, 'wb')
            fh.write(contents or '')
            fh.close()
            fh = self.fake_open(fake_path, 'wb')
            fh.write(contents or '')
            fh.close()
        # l for symlink, h for hard link
        if file_type in ('l', 'h'):
            real_target, fake_target = (contents, contents)
            # If it begins with '/', make it relative to the base. You can't go
            # creating files in / for the real file system.
            if contents.startswith(os.sep):
                real_target, fake_target = self._paths(contents[1:])
            if file_type == 'l':
                os.symlink(real_target, real_path)
                self.fake_os.symlink(fake_target, fake_path)
            elif file_type == 'h':
                os.link(real_target, real_path)
                self.fake_os.link(fake_target, fake_path)

    def setUp(self):
        # Base paths in the real and test file systems. We keep them different
        # so that missing features in the fake don't fall through to the base
        # operations and magically succeed.
        tsname = 'fakefs.%s' % time.time()
        self.cwd = os.getcwd()
        # Fully expand the base_path - required on OS X.
        self.real_base = os.path.realpath(
            os.path.join(tempfile.gettempdir(), tsname))
        os.chdir(tempfile.gettempdir())
        if os.path.isdir(self.real_base):
            shutil.rmtree(self.real_base)
        os.mkdir(self.real_base)
        self.fake_base = self._FAKE_FS_BASE

        # Make sure we can write to the physical testing temp directory.
        self.assertTrue(os.access(self.real_base, os.W_OK))

        self.fake_filesystem = fake_filesystem.FakeFilesystem()
        self.fake_filesystem.create_dir(self.fake_base)
        self.fake_os = fake_filesystem.FakeOsModule(self.fake_filesystem)
        self.fake_open = fake_filesystem.FakeFileOpen(self.fake_filesystem)
        self._created_files = []

        os.chdir(self.real_base)
        self.fake_os.chdir(self.fake_base)

    def tearDown(self):
        # We have to remove all the files from the real FS. Doing the same for
        # the fake FS is optional, but doing it is an extra sanity check.
        os.chdir(tempfile.gettempdir())
        try:
            rev_files = self._created_files[:]
            rev_files.reverse()
            for info in rev_files:
                real_path, fake_path = self._paths(info[1])
                if info[0] == 'd':
                    try:
                        os.rmdir(real_path)
                    except OSError as e:
                        if 'Directory not empty' in e:
                            self.fail('Real path %s not empty: %s : %s' % (
                                real_path, e, os.listdir(real_path)))
                        else:
                            raise
                    self.fake_os.rmdir(fake_path)
                if info[0] == 'f' or info[0] == 'l':
                    os.remove(real_path)
                    self.fake_os.remove(fake_path)
        finally:
            shutil.rmtree(self.real_base)
            os.chdir(self.cwd)

    def _compare_behaviors(self, method_name, path, real, fake,
                           method_returns_path=False):
        """Invoke an os method in both real and fake contexts and compare
        results.

        Invoke a real filesystem method with a path to a real file and invoke
        a fake filesystem method with a path to a fake file and compare the
        results. We expect some calls to throw Exceptions, so we catch those
        and compare them.

        Args:
            method_name: Name of method being tested, for use in
                error messages.
            path: potential path to a file in the real and fake file systems,
                passing an empty tuple indicates that no arguments to pass
                to method.
            real: built-in system library or method from the built-in system
                library which takes a path as an arg and returns some value.
            fake: fake_filsystem object or method from a fake_filesystem class
                which takes a path as an arg and returns some value.
            method_returns_path: True if the method returns a path, and thus we
                must compensate for expected difference between real and fake.

        Returns:
            A description of the difference in behavior, or None.
        """
        # pylint: disable=C6403

        def _error_class(exc):
            if exc:
                if hasattr(exc, 'errno'):
                    return '{}({})'.format(exc.__class__.__name__, exc.errno)
                return exc.__class__.__name__
            return 'None'

        real_err, real_value = self._get_real_value(method_name, path, real)
        fake_err, fake_value = self._get_fake_value(method_name, path, fake)

        method_call = '%s' % method_name
        method_call += '()' if path == () else '(%s)' % path
        # We only compare on the error class because the acutal error contents
        # is almost always different because of the file paths.
        if _error_class(real_err) != _error_class(fake_err):
            if real_err is None:
                return '%s: real version returned %s, fake raised %s' % (
                    method_call, real_value, _error_class(fake_err))
            if fake_err is None:
                return '%s: real version raised %s, fake returned %s' % (
                    method_call, _error_class(real_err), fake_value)
            return '%s: real version raised %s, fake raised %s' % (
                method_call, _error_class(real_err), _error_class(fake_err))
        real_errno = _get_errno(real_err)
        fake_errno = _get_errno(fake_err)
        if real_errno != fake_errno:
            return '%s(%s): both raised %s, real errno %s, fake errno %s' % (
                method_name, path, _error_class(real_err),
                real_errno, fake_errno)
        # If the method is supposed to return a full path AND both values
        # begin with the expected full path, then trim it off.
        if method_returns_path:
            if (real_value and fake_value
                    and real_value.startswith(self.real_base)
                    and fake_value.startswith(self.fake_base)):
                real_value = real_value[len(self.real_base):]
                fake_value = fake_value[len(self.fake_base):]
        if real_value != fake_value:
            return '%s: real return %s, fake returned %s' % (
                method_call, real_value, fake_value)
        return None

    @staticmethod
    def _get_fake_value(method_name, path, fake):
        fake_value = None
        fake_err = None
        try:
            fake_method = fake
            if not callable(fake):
                fake_method = getattr(fake, method_name)
            args = [] if path == () else [path]
            fake_value = str(fake_method(*args))
        except Exception as e:  # pylint: disable-msg=W0703
            fake_err = e
        return fake_err, fake_value

    @staticmethod
    def _get_real_value(method_name, path, real):
        real_value = None
        real_err = None
        # Catching Exception below gives a lint warning, but it's what we need.
        try:
            args = [] if path == () else [path]
            real_method = real
            if not callable(real):
                real_method = getattr(real, method_name)
            real_value = str(real_method(*args))
        except Exception as e:  # pylint: disable-msg=W0703
            real_err = e
        return real_err, real_value

    def assertOsMethodBehaviorMatches(self, method_name, path,
                                      method_returns_path=False):
        """Invoke an os method in both real and fake contexts and compare.

        For a given method name (from the os module) and a path, compare the
        behavior of the system provided module against the fake_filesystem
        module.
        We expect results and/or Exceptions raised to be identical.

        Args:
            method_name: Name of method being tested.
            path: potential path to a file in the real and fake file systems.
            method_returns_path: True if the method returns a path, and thus we
                must compensate for expected difference between real and fake.

        Returns:
            A description of the difference in behavior, or None.
        """
        path = sep(path)
        return self._compare_behaviors(method_name, path, os, self.fake_os,
                                       method_returns_path)

    def diff_open_method_behavior(self, method_name, path, mode, data,
                                  method_returns_data=True):
        """Invoke an open method in both real and fkae contexts and compare.

        Args:
            method_name: Name of method being tested.
            path: potential path to a file in the real and fake file systems.
            mode: how to open the file.
            data: any data to pass to the method.
            method_returns_data: True if a method returns some sort of data.

        For a given method name (from builtin open) and a path, compare the
        behavior of the system provided module against the fake_filesystem
        module.
        We expect results and/or Exceptions raised to be identical.

        Returns:
            A description of the difference in behavior, or None.
        """
        with open(path, mode) as real_fh:
            with self.fake_open(path, mode) as fake_fh:
                return self._compare_behaviors(
                    method_name, data, real_fh, fake_fh, method_returns_data)

    def diff_os_path_method_behavior(self, method_name, path,
                                     method_returns_path=False):
        """Invoke an os.path method in both real and fake contexts and compare.

        For a given method name (from the os.path module) and a path, compare
        the behavior of the system provided module against the
        fake_filesytem module.
        We expect results and/or Exceptions raised to be identical.

        Args:
            method_name: Name of method being tested.
            path: potential path to a file in the real and fake file systems.
            method_returns_path: True if the method returns a path, and thus we
                must compensate for expected difference between real and fake.

        Returns:
            A description of the difference in behavior, or None.
        """
        return self._compare_behaviors(method_name, path, os.path,
                                       self.fake_os.path,
                                       method_returns_path)

    def assertOsPathMethodBehaviorMatches(self, method_name, path,
                                          method_returns_path=False):
        """Assert that an os.path behaves the same in both real and
        fake contexts.

        Wraps DiffOsPathMethodBehavior, raising AssertionError if any
        differences are reported.

        Args:
            method_name: Name of method being tested.
            path: potential path to a file in the real and fake file systems.
            method_returns_path: True if the method returns a path, and thus we
                must compensate for expected difference between real and fake.

        Raises:
            AssertionError if there is any difference in behavior.
        """
        path = sep(path)
        diff = self.diff_os_path_method_behavior(
            method_name, path, method_returns_path)
        if diff:
            self.fail(diff)

    def assertAllOsBehaviorsMatch(self, path):
        path = sep(path)
        os_method_names = [] if self.is_windows else ['readlink']
        os_method_names_no_args = ['getcwd']
        os_path_method_names = ['isabs',
                                'isdir',
                                'isfile',
                                'exists'
                                ]
        if not self.is_windows:
            os_path_method_names.append('islink')
            os_path_method_names.append('lexists')
        wrapped_methods = [
            ['access', self._access_real, self._access_fake],
            ['stat.size', self._stat_size_real, self._stat_size_fake],
            ['lstat.size', self._lstat_size_real, self._lstat_size_fake]
        ]

        differences = []
        for method_name in os_method_names:
            diff = self.assertOsMethodBehaviorMatches(method_name, path)
            if diff:
                differences.append(diff)
        for method_name in os_method_names_no_args:
            diff = self.assertOsMethodBehaviorMatches(method_name, (),
                                                      method_returns_path=True)
            if diff:
                differences.append(diff)
        for method_name in os_path_method_names:
            diff = self.diff_os_path_method_behavior(method_name, path)
            if diff:
                differences.append(diff)
        for m in wrapped_methods:
            diff = self._compare_behaviors(m[0], path, m[1], m[2])
            if diff:
                differences.append(diff)
        if differences:
            self.fail('Behaviors do not match for %s:\n    %s' %
                      (path, '\n    '.join(differences)))

    def assertFileHandleBehaviorsMatch(self, path, mode, data):
        path = sep(path)
        write_method_names = ['write', 'writelines']
        read_method_names = ['read', 'readlines']
        other_method_names = ['truncate', 'flush', 'close']
        differences = []
        for method_name in write_method_names:
            diff = self.diff_open_method_behavior(
                method_name, path, mode, data)
            if diff:
                differences.append(diff)
        for method_name in read_method_names + other_method_names:
            diff = self.diff_open_method_behavior(method_name, path, mode, ())
            if diff:
                differences.append(diff)
        if differences:
            self.fail('Behaviors do not match for %s:\n    %s' %
                      (path, '\n    '.join(differences)))

    # Helpers for checks which are not straight method calls.
    @staticmethod
    def _access_real(path):
        return os.access(path, os.R_OK)

    def _access_fake(self, path):
        return self.fake_os.access(path, os.R_OK)

    def _stat_size_real(self, path):
        real_path, unused_fake_path = self._paths(path)
        # fake_filesystem.py does not implement stat().st_size for directories
        if os.path.isdir(real_path):
            return None
        return os.stat(real_path).st_size

    def _stat_size_fake(self, path):
        unused_real_path, fake_path = self._paths(path)
        # fake_filesystem.py does not implement stat().st_size for directories
        if self.fake_os.path.isdir(fake_path):
            return None
        return self.fake_os.stat(fake_path).st_size

    def _lstat_size_real(self, path):
        real_path, unused_fake_path = self._paths(path)
        if os.path.isdir(real_path):
            return None
        size = os.lstat(real_path).st_size
        # Account for the difference in the lengths of the absolute paths.
        if os.path.islink(real_path):
            if os.readlink(real_path).startswith(os.sep):
                size -= len(self.real_base)
        return size

    def _lstat_size_fake(self, path):
        unused_real_path, fake_path = self._paths(path)
        # size = 0
        if self.fake_os.path.isdir(fake_path):
            return None
        size = self.fake_os.lstat(fake_path).st_size
        # Account for the difference in the lengths of the absolute paths.
        if self.fake_os.path.islink(fake_path):
            if self.fake_os.readlink(fake_path).startswith(os.sep):
                size -= len(self.fake_base)
        return size

    def test_isabs(self):
        # We do not have to create any files for isabs.
        self.assertOsPathMethodBehaviorMatches('isabs', None)
        self.assertOsPathMethodBehaviorMatches('isabs', '')
        self.assertOsPathMethodBehaviorMatches('isabs', '/')
        self.assertOsPathMethodBehaviorMatches('isabs', '/a')
        self.assertOsPathMethodBehaviorMatches('isabs', 'a')

    def test_none_path(self):
        self.assertAllOsBehaviorsMatch(None)

    def test_empty_path(self):
        self.assertAllOsBehaviorsMatch('')

    def test_root_path(self):
        self.assertAllOsBehaviorsMatch('/')

    def test_non_existant_file(self):
        self.assertAllOsBehaviorsMatch('foo')

    def test_empty_file(self):
        self._create_test_file('f', 'aFile')
        self.assertAllOsBehaviorsMatch('aFile')

    def test_file_with_contents(self):
        self._create_test_file('f', 'aFile', 'some contents')
        self.assertAllOsBehaviorsMatch('aFile')

    def test_file_with_binary_contents(self):
        self._create_test_file('b', 'aFile', b'some contents')
        self.assertAllOsBehaviorsMatch('aFile')

    @unittest.skipIf(TestCase.is_windows, 'no symlink in Windows')
    def test_sym_link_to_empty_file(self):
        self._create_test_file('f', 'aFile')
        self._create_test_file('l', 'link_to_empty', 'aFile')
        self.assertAllOsBehaviorsMatch('link_to_empty')

    @unittest.skipIf(TestCase.is_windows, 'no symlink in Windows')
    def test_hard_link_to_empty_file(self):
        self._create_test_file('f', 'aFile')
        self._create_test_file('h', 'link_to_empty', 'aFile')
        self.assertAllOsBehaviorsMatch('link_to_empty')

    @unittest.skipIf(TestCase.is_windows, 'no symlink in Windows')
    def test_sym_link_to_real_file(self):
        self._create_test_file('f', 'aFile', 'some contents')
        self._create_test_file('l', 'link_to_file', 'aFile')
        self.assertAllOsBehaviorsMatch('link_to_file')

    @unittest.skipIf(TestCase.is_windows, 'no symlink in Windows')
    def test_hard_link_to_real_file(self):
        self._create_test_file('f', 'aFile', 'some contents')
        self._create_test_file('h', 'link_to_file', 'aFile')
        self.assertAllOsBehaviorsMatch('link_to_file')

    @unittest.skipIf(TestCase.is_windows, 'no symlink in Windows')
    def test_broken_sym_link(self):
        self._create_test_file('l', 'broken_link', 'broken')
        self._create_test_file('l', 'loop', '/a/loop')
        self.assertAllOsBehaviorsMatch('broken_link')

    def test_file_in_a_folder(self):
        self._create_test_file('d', 'a')
        self._create_test_file('d', 'a/b')
        self._create_test_file('f', 'a/b/file', 'contents')
        self.assertAllOsBehaviorsMatch('a/b/file')

    @unittest.skipIf(TestCase.is_windows, 'no symlink in Windows')
    def test_absolute_sym_link_to_folder(self):
        self._create_test_file('d', 'a')
        self._create_test_file('d', 'a/b')
        self._create_test_file('f', 'a/b/file', 'contents')
        self._create_test_file('l', 'a/link', '/a/b')
        self.assertAllOsBehaviorsMatch('a/link/file')

    @unittest.skipIf(TestCase.is_windows, 'no symlink in Windows')
    def test_link_to_folder_after_chdir(self):
        self._create_test_file('d', 'a')
        self._create_test_file('d', 'a/b')
        self._create_test_file('f', 'a/b/file', 'contents')
        self._create_test_file('l', 'a/link', '/a/b')

        real_dir, fake_dir = self._paths('a/b')
        os.chdir(real_dir)
        self.fake_os.chdir(fake_dir)
        self.assertAllOsBehaviorsMatch('file')

    @unittest.skipIf(TestCase.is_windows, 'no symlink in Windows')
    def test_relative_sym_link_to_folder(self):
        self._create_test_file('d', 'a')
        self._create_test_file('d', 'a/b')
        self._create_test_file('f', 'a/b/file', 'contents')
        self._create_test_file('l', 'a/link', 'b')
        self.assertAllOsBehaviorsMatch('a/link/file')

    @unittest.skipIf(TestCase.is_windows, 'no symlink in Windows')
    def test_sym_link_to_parent(self):
        # Soft links on HFS+ / OS X behave differently.
        if os.uname()[0] != 'Darwin':
            self._create_test_file('d', 'a')
            self._create_test_file('d', 'a/b')
            self._create_test_file('l', 'a/b/c', '..')
            self.assertAllOsBehaviorsMatch('a/b/c')

    @unittest.skipIf(TestCase.is_windows, 'no symlink in Windows')
    def test_path_through_sym_link_to_parent(self):
        self._create_test_file('d', 'a')
        self._create_test_file('f', 'a/target', 'contents')
        self._create_test_file('d', 'a/b')
        self._create_test_file('l', 'a/b/c', '..')
        self.assertAllOsBehaviorsMatch('a/b/c/target')

    @unittest.skipIf(TestCase.is_windows, 'no symlink in Windows')
    def test_sym_link_to_sibling_directory(self):
        self._create_test_file('d', 'a')
        self._create_test_file('d', 'a/b')
        self._create_test_file('d', 'a/sibling_of_b')
        self._create_test_file('f', 'a/sibling_of_b/target', 'contents')
        self._create_test_file('l', 'a/b/c', '../sibling_of_b')
        self.assertAllOsBehaviorsMatch('a/b/c/target')

    @unittest.skipIf(TestCase.is_windows, 'no symlink in Windows')
    def test_sym_link_to_sibling_directory_non_existant_file(self):
        self._create_test_file('d', 'a')
        self._create_test_file('d', 'a/b')
        self._create_test_file('d', 'a/sibling_of_b')
        self._create_test_file('f', 'a/sibling_of_b/target', 'contents')
        self._create_test_file('l', 'a/b/c', '../sibling_of_b')
        self.assertAllOsBehaviorsMatch('a/b/c/file_does_not_exist')

    @unittest.skipIf(TestCase.is_windows, 'no symlink in Windows')
    def test_broken_sym_link_to_sibling_directory(self):
        self._create_test_file('d', 'a')
        self._create_test_file('d', 'a/b')
        self._create_test_file('d', 'a/sibling_of_b')
        self._create_test_file('f', 'a/sibling_of_b/target', 'contents')
        self._create_test_file('l', 'a/b/c', '../broken_sibling_of_b')
        self.assertAllOsBehaviorsMatch('a/b/c/target')

    def test_relative_path(self):
        self._create_test_file('d', 'a')
        self._create_test_file('d', 'a/b')
        self._create_test_file('d', 'a/sibling_of_b')
        self._create_test_file('f', 'a/sibling_of_b/target', 'contents')
        self.assertAllOsBehaviorsMatch('a/b/../sibling_of_b/target')

    def test_broken_relative_path(self):
        self._create_test_file('d', 'a')
        self._create_test_file('d', 'a/b')
        self._create_test_file('d', 'a/sibling_of_b')
        self._create_test_file('f', 'a/sibling_of_b/target', 'contents')
        self.assertAllOsBehaviorsMatch('a/b/../broken/target')

    def test_bad_relative_path(self):
        self._create_test_file('d', 'a')
        self._create_test_file('f', 'a/target', 'contents')
        self._create_test_file('d', 'a/b')
        self._create_test_file('d', 'a/sibling_of_b')
        self._create_test_file('f', 'a/sibling_of_b/target', 'contents')
        self.assertAllOsBehaviorsMatch('a/b/../broken/../target')

    def test_getmtime_nonexistant_path(self):
        self.assertOsPathMethodBehaviorMatches('getmtime', 'no/such/path')

    def test_builtin_open_modes(self):
        self._create_test_file('f', 'read', 'some contents')
        self._create_test_file('f', 'write', 'some contents')
        self._create_test_file('f', 'append', 'some contents')
        self.assertFileHandleBehaviorsMatch('read', 'r', 'other contents')
        self.assertFileHandleBehaviorsMatch('write', 'w', 'other contents')
        self.assertFileHandleBehaviorsMatch('append', 'a', 'other contents')
        self._create_test_file('f', 'readplus', 'some contents')
        self._create_test_file('f', 'writeplus', 'some contents')
        self.assertFileHandleBehaviorsMatch(
            'readplus', 'r+', 'other contents')
        self.assertFileHandleBehaviorsMatch(
            'writeplus', 'w+', 'other contents')
        self._create_test_file('b', 'binaryread', b'some contents')
        self._create_test_file('b', 'binarywrite', b'some contents')
        self._create_test_file('b', 'binaryappend', b'some contents')
        self.assertFileHandleBehaviorsMatch(
            'binaryread', 'rb', b'other contents')
        self.assertFileHandleBehaviorsMatch(
            'binarywrite', 'wb', b'other contents')
        self.assertFileHandleBehaviorsMatch(
            'binaryappend', 'ab', b'other contents')
        self.assertFileHandleBehaviorsMatch('read', 'rb', 'other contents')
        self.assertFileHandleBehaviorsMatch('write', 'wb', 'other contents')
        self.assertFileHandleBehaviorsMatch('append', 'ab', 'other contents')


def main(_):
    unittest.main()


if __name__ == '__main__':
    unittest.main()
